#ifndef __DBUTILS_H_
#include <db4/db.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "dbutils.h"
#include "alis.h"

DBC *bdb_dbcp;
DB *channel_database;
int totalchannels;

int fexists (char *filename)
{
        struct stat buf;
        return stat(filename, &buf);
}


DB *bdb_open (char *filename)
{
        int ret;
        DB *dbp;

        dbp = NULL;

        ret = db_create(&dbp, NULL, 0);
        if (ret != 0) {
        fprintf(stderr,
        "db_create: %s\n", db_strerror(ret));
        return NULL;
        }
        dbp->set_errfile(dbp, stderr);
        dbp->set_errpfx(dbp, "BDB");


/*        if (fexists(filename) == 0)
                ret = dbp->open(dbp,NULL, filename, NULL, DB_BTREE, DB_THREAD, 0664);
        else
                ret = dbp->open(dbp,NULL, filename, NULL, DB_BTREE, DB_CREATE|DB_THREAD, 0664);
*/
		ret = dbp->open(dbp,NULL, NULL, NULL, DB_BTREE, DB_CREATE|DB_THREAD, 0664);

        if (ret != 0)
        {
                dbp->err(dbp, ret, "Error opening database %s",filename);
                return NULL;
        }
		totalchannels = 0;
        return dbp;
}


void bdb_close (DB *b)
{
        b->sync(b,0);
        b->close(b, 0);
}

DBT bdb_fetch (DB *dbfile, DBT key)
{
        DBT result;
        int res;

        memset(&result, 0, sizeof(DBT));
        result.flags = DB_DBT_MALLOC;
        key.flags = DB_DBT_USERMEM;

        res = dbfile->get(dbfile, NULL, &key, &result, 0);
        if (res != 0)
        {
                if (res != DB_NOTFOUND)
                dbfile->err(dbfile, res, "");
        }

   return result;
}

DBT bdb_firstkey (DB *dbp)
{
    int ret;
    DBT key;
        DBT data;

    memset(&key, 0, sizeof(DBT));
    memset(&data, 0, sizeof(DBT));


	bdb_dbcp = NULL;

    if ((ret = dbp->cursor(dbp, NULL, &bdb_dbcp, 0)) != 0) {
                dbp->err(dbp, ret, "DB->cursor");
        }

        ret = bdb_dbcp->c_get(bdb_dbcp, &key, &data, DB_NEXT);
        return key;
}

int bdb_nextkey (DBT *key)
{
    DBT data;

    memset(&data, 0, sizeof(DBT));

    return bdb_dbcp->c_get(bdb_dbcp, key, &data, DB_NEXT);
}

int bdb_delete (DB *dbfile, DBT key)
{
        if ((dbfile->del(dbfile,NULL,&key,0)) != 0)
                return 1;
        else
        return 0;
}
int bdb_store (DB *a, DBT b, DBT c)
{
        int ret;

        switch (ret = a->put(a, NULL, &b, &c, 0))
        {
        case 0:
                break;
        default:
                a->err(a, ret, "DB->put");
                if (ret != DB_KEYEXIST)
                {
                        printf("DB4: Error when writing to database\n");
                }
                break;
        }
		a->sync(a,0);

        return 0;
}

int find_in_bdb_by_name(chaninfo *info , char *channelname)
{
	DBT content, key2; 
	size_t mysize;

	memset(&content, 0, sizeof(DBT));
	memset(&key2, 0, sizeof(DBT));

	key2.data = channelname;
	key2.size = strlen(channelname)+1; 

	content = bdb_fetch(channel_database, key2); 

	if (content.data)
   	{
		char *p = (char *) content.data;

		memcpy(&mysize, p, sizeof(size_t)); 
       	p += sizeof(size_t); 
       	memcpy(info->topic, p, mysize); 
		p+= mysize;

		memcpy(&mysize, p, sizeof(size_t)); 
       	p += sizeof(size_t); 
		memcpy(info->topicnick, p, mysize); 
		p+= mysize;

		memcpy(&mysize, p, sizeof(size_t)); 
       	p += sizeof(size_t); 
		memcpy(info->mode, p, mysize); 
		p+= mysize;
		
        memcpy(&info->members, p, sizeof(long));
		
		strcpy(info->name,channelname);

		return RC_BDB_FOUND;
	}

	return RC_BDB_NOTFOUND;
}

void remove_from_bdb (char *channelname)
{
	DBT key;

    memset(&key, 0, sizeof(DBT));

	key.data = channelname;
	key.size = strlen(channelname)+1;

	bdb_delete(channel_database,key);

	/* channel_database->sync(channel_database,0);*/

	return;
}

void add_to_bdb(char *channelname, chaninfo *tmp)
{

    DBT key, content;

	char *p;
	size_t size;
	int totalsize;
	char buf[1024];
    memset(&key, 0, sizeof(DBT));
    memset(&content, 0, sizeof(DBT));

    key.data = channelname;
    key.size = strlen(channelname)+1;

    content.size = 0;
	content.data = buf;
	p = (char *)content.data;

    totalsize = 0;

    size = strlen(tmp->topic)+1;
    totalsize += size;
    memcpy(p, &size, sizeof(size_t)); p += sizeof(size_t);
	totalsize += sizeof(size_t);
    memcpy(p, tmp->topic, size); p += size;

    size = strlen(tmp->topicnick)+1;
    totalsize += size;
    memcpy(p, &size, sizeof(size_t)); p += sizeof(size_t);
	totalsize += sizeof(size_t);
    memcpy(p, tmp->topicnick, size); p += size;

    size = strlen(tmp->mode)+1;
    totalsize += size;
    memcpy(p, &size, sizeof(size_t)); p += sizeof(size_t);
	totalsize += sizeof(size_t);
    memcpy(p, tmp->mode, size); p += size;

    totalsize += sizeof(long);
    memcpy(p, &tmp->members, sizeof(long)); p += sizeof(long);

    content.size = totalsize;

	bdb_store(channel_database,key,content);

/*    channel_database->sync(channel_database,0);*/

	return;
}


int bdb_next_channel (chaninfo *tmp)
{
	DBT key;
	int ret;

    ret = bdb_nextkey(&key);

	if (ret == 0)
		return find_in_bdb_by_name(tmp,(char *)key.data);		

    return RC_BDB_NOTFOUND;
}

int bdb_first_channel (chaninfo *tmp)
{

  DBT key;

  memset(&key, 0, sizeof(DBT));

  bdb_dbcp = NULL;
  key = bdb_firstkey(channel_database);
  if (key.data) 
    return find_in_bdb_by_name(tmp,(char *)key.data);

    return RC_BDB_NOTFOUND;
}

#endif
